/*
 * Copyright (C) 2004 emuWorks
 * http://games.technoplaza.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <wx/wxprec.h>

#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

#include <iostream>
#include <fstream>
#include <vector>

#include <sys/stat.h>

#include "chtdecoder.hh"
#include "datastructures/chtcode.hh"
#include "exceptions/invalidchtfileexception.hh"
#include "tools/chtfilereader.hh"
#include "view/chtframe.hh"

// static variable initialization
wxString *CHTDecoder::APP_NAME = new wxString("wxCHTDecoder");
wxString *CHTDecoder::APP_VERSION = new wxString("1.1");
wxString *CHTDecoder::APP_COPYRIGHT = 
    new wxString("Copyright (C) 2004 emuWorks");

wxSize *CHTDecoder::APP_SIZE = new wxSize(640, 480);

// macro to generate the statup code using CHTDecoder as the starting class
IMPLEMENT_APP(CHTDecoder);

void CHTDecoder::runTextMode() {
    CHTFileReader &reader = CHTFileReader::getInstance();
    struct stat filestats;
    
    for (int i = 1; i < argc; i++) {
        try {
            wxString filename(argv[i]);
            std::vector<CHTCode> &codes = reader.readCHTFile(filename);
            
            filename += ".txt";
            
            std::ofstream *out = NULL;
            std::ostream *output = &std::cout;
            
            if (stat(filename.c_str(), &filestats) == 0) {
                std::cerr << "error: " << filename.c_str() << 
                    " already exists. Will output to console instead.\n";
            } else {
                out = new std::ofstream(filename.c_str());
                
                if (!(*out)) {
                    std::cerr << "error: unable to open " << filename <<
                        " for writing. Will output to console instead.\n";
                    
                    delete out;
                    out = NULL;
                } else {
                    output = out;
                }
            }
            
            for (size_t j = 0; j < codes.size(); j++) {
                CHTCode &code = codes[j];
                
                wxString str;
                str.Alloc(100);
                str.Printf("%2d: %s\n", (j + 1), code.toString().c_str());
                
                (*output) << str.c_str();
            }
            
            if (out) {
                std::cout << "Wrote the decoded cheats to " << filename.c_str() 
                    << '.' << '\n';
                
                out->close();
                delete out;
            }
        } catch (InvalidCHTFileException e) {
            std::cerr << "error: " << argv[i] << " is not a valid .CHT file.\n";
        }
    }
}

bool CHTDecoder::OnInit() {
    if (argc > 1) {
        runTextMode();
        
        return false;
    }
    
    CHTFrame *frame = new CHTFrame(*APP_NAME + ' ' + *APP_VERSION,
                                   wxPoint(0, 0),
                                   *APP_SIZE);
    
    frame->Show(true);
    SetTopWindow(frame);
    
    return true;
}

